@@ -11,6 +11,7 @@ __all__ = ["maximize", "optimize", "optimize_iterator", "with_input_noise"]
 
 ArrayLike = Union[
     np.ndarray,
+    float,
     Sequence[float],
     Sequence[Sequence[float]],
     Sequence[Sequence[Sequence[float]]],
@@ -54,8 +55,8 @@ def _type_check(
     # Type-check.
     if not callable(f):
         raise TypeError(f"f must be callable, got {f!r}")
-    elif not isinstance(x, (np.ndarray, Sequence)):
-        raise TypeError(f"x must be either a numpy array or a sequence, got {x!r}")
+    elif not isinstance(x, (float, int, np.ndarray, Sequence)):
+        raise TypeError(f"x must be either a real number, numpy array, or sequence, got {x!r}")
     elif not isinstance(iterations, int):
         raise TypeError(f"iterations must be an integer, got {iterations!r}")
     elif not isinstance(adam, SupportsIndex):
@@ -105,6 +106,8 @@ def maximize(f: Callable[[np.ndarray], float], /) -> Callable[[np.ndarray], floa
         Maximize a function instead of minimizing it:
             x = spsa.optimize(maximize(f), x)
     """
+    if not callable(f):
+        raise TypeError(f"f must be callable, got {f!r}")
     @wraps(f)
     def wrapper(x: np.ndarray, /) -> float:
         return -f(x)
@@ -124,11 +127,13 @@ def with_input_noise(f: Callable[[np.ndarray], float], /, noise: float) -> Calla
             yield random_noise
             yield random_noise
     rng_iter: Optional[Iterator[float]] = None
+    @wraps(f)
     def wrapper(x: np.ndarray) -> float:
         nonlocal rng_iter
         if rng_iter is None:
             rng_iter = rng_iterator(x.shape)
-        return f(x + next(rng_iter))
+        dx = next(rng_iter)
+        return (f(x + dx) + f(x - dx)) / 2
     return wrapper
 
 def optimize(
@@ -415,8 +420,8 @@ def optimize(
             dx /= np.sqrt(square_gx / b2 + epsilon)
         # Sample points.
         y3 = f(x)
-        y4 = f(x - lr / 3 * dx)
-        y5 = f(x - lr * 3 * dx)
+        y4 = f(x - lr * m1 * dx)
+        y5 = f(x - lr / m1 * dx)
         y6 = f(x)
         # Estimate the noise in f.
         bn += m2 * (1 - bn)
@@ -645,13 +650,13 @@ def optimize_iterator(
     bx = mx
     x_avg = mx * x
     # Track the best (x, y).
-    y_min = y
+    y_min = y / bn
     x_min = x.copy()
     # Track how many times the solution fails to improve.
     consecutive_fails = 0
     improvement_fails = 0
     # Generate initial iteration.
-    variables = dict(
+    yield dict(
         x_min=x_min,
         y_min=y_min,
         x=x_avg,
@@ -666,8 +671,6 @@ def optimize_iterator(
         slow_gradient=slow_gx,
         square_gradient=square_gx,
     )
-    yield variables
-    del variables
     # Initial step size.
     dx = gx / b1
     if adam:
@@ -696,8 +699,8 @@ def optimize_iterator(
             dx /= np.sqrt(square_gx / b2 + epsilon)
         # Sample points.
         y3 = f(x)
-        y4 = f(x - lr / 3 * dx)
-        y5 = f(x - lr * 3 * dx)
+        y4 = f(x - lr * m1 * dx)
+        y5 = f(x - lr / m1 * dx)
         y6 = f(x)
         # Estimate the noise in f.
         bn += m2 * (1 - bn)
@@ -731,7 +734,7 @@ def optimize_iterator(
             x_min = x_avg / bx
             consecutive_fails = 0
         # Generate the variables for the next iteration.
-        variables = dict(
+        yield dict(
             x_min=x_min,
             y_min=y_min,
             x=x_avg,
@@ -746,8 +749,6 @@ def optimize_iterator(
             slow_gradient=slow_gx,
             square_gradient=square_gx,
         )
-        yield variables
-        del variables
         if consecutive_fails < 128 * (improvement_fails + isqrt(x.size + 100)):
             continue
         # Reset variables if diverging.
