@@ -1,10 +1,21 @@
 """
-Contains asynchronous variants of `spsa.optimize` and `spsa.optimize_iterator`.
+Asynchronous IO
+----------------
+
+Contains asynchronous variants of `spsa.optimize` and `spsa.optimize_iterator`
+for calling an asynchronous function concurrently each iteration.
+
+See also:
+    spsa.amp - Asynchronous Multiprocessing:
+        Runs the whole spsa algorithm in a separate process for synchronous functions.
+        Unlike spsa.aio, does not concurrently call functions each iteration. This is
+        more appropriate if the objective function cannot be ran concurrently or if
+        sharing numpy arrays between processes is too expensive.
 
 Example Uses
 -------------
 - Large numbers of iterations can be ran concurrently with other asynchronous code.
-- Slow function calls can be ran concurrently each iteration. May be combined with multiprocessing.
+- Slow function calls can be ran concurrently each iteration by combining it with multiprocessing.
 
 Parallelizing Calls with Multiprocessing
 -----------------------------------------
@@ -58,6 +69,8 @@ async def maximize(f: Callable[[np.ndarray], Awaitable[float]], /) -> Callable[[
         Maximize a function instead of minimizing it:
             x = spsa.aio.optimize(maximize(f), x)
     """
+    if not callable(f):
+        raise TypeError(f"f must be callable, got {f!r}")
     @wraps(f)
     async def wrapper(x: np.ndarray, /) -> float:
         return -(await f(x))
@@ -81,7 +94,7 @@ def with_input_noise(f: Callable[[np.ndarray], Awaitable[float]], /, noise: floa
         nonlocal rng_iter
         if rng_iter is None:
             rng_iter = rng_iterator(x.shape)
-        return await f(x + next(rng_iter))
+        return (await f(x + dx) + await f(x - dx)) / 2
     return wrapper
 
 async def optimize(
@@ -213,7 +226,7 @@ async def optimize(
     bx = mx
     x_avg = mx * x
     # Track the best (x, y).
-    y_min = y
+    y_min = y / bn
     x_min = x.copy()
     # Track how many times the solution fails to improve.
     consecutive_fails = 0
@@ -244,7 +257,7 @@ async def optimize(
         if adam:
             dx /= np.sqrt(square_gx / b2 + epsilon)
         # Sample points concurrently.
-        y3, y4, y5, y6 = await asyncio.gather(f(x), f(x - lr / 3 * dx), f(x - lr * 3 * dx), f(x))
+        y3, y4, y5, y6 = await asyncio.gather(f(x), f(x - lr * m1 * dx), f(x - lr / m1 * dx), f(x))
         # Estimate the noise in f.
         bn += m2 * (1 - bn)
         y += m2 * (y3 - y)
@@ -425,13 +438,13 @@ async def optimize_iterator(
     bx = mx
     x_avg = mx * x
     # Track the best (x, y).
-    y_min = y
+    y_min = y / bn
     x_min = x.copy()
     # Track how many times the solution fails to improve.
     consecutive_fails = 0
     improvement_fails = 0
     # Generate initial iteration.
-    variables = dict(
+    yield dict(
         x_min=x_min,
         y_min=y_min,
         x=x_avg,
@@ -446,8 +459,6 @@ async def optimize_iterator(
         slow_gradient=slow_gx,
         square_gradient=square_gx,
     )
-    yield variables
-    del variables
     # Initial step size.
     dx = gx / b1
     if adam:
@@ -474,7 +485,7 @@ async def optimize_iterator(
         if adam:
             dx /= np.sqrt(square_gx / b2 + epsilon)
         # Sample points concurrently.
-        y3, y4, y5, y6 = await asyncio.gather(f(x), f(x - lr / 3 * dx), f(x - lr * 3 * dx), f(x))
+        y3, y4, y5, y6 = await asyncio.gather(f(x), f(x - lr * m1 * dx), f(x - lr / m1 * dx), f(x))
         # Estimate the noise in f.
         bn += m2 * (1 - bn)
         y += m2 * (y3 - y)
@@ -507,7 +518,7 @@ async def optimize_iterator(
             x_min = x_avg / bx
             consecutive_fails = 0
         # Generate the variables for the next iteration.
-        variables = dict(
+        yield dict(
             x_min=x_min,
             y_min=y_min,
             x=x_avg,
@@ -522,8 +533,6 @@ async def optimize_iterator(
             slow_gradient=slow_gx,
             square_gradient=square_gx,
         )
-        yield variables
-        del variables
         await asyncio.sleep(0)
         if consecutive_fails < 128 * (improvement_fails + isqrt(x.size + 100)):
             continue
